import sqlite3
from typing import List, Tuple
from nonebot.log import logger
from os.path import abspath, dirname
from .exception import *

class BiliDatabase():
    def __init__(self) -> None:
        self.db_name = dirname(abspath(__file__)) + "/../bilibili_2.db"
        self.conn = sqlite3.connect(self.db_name)
        self.init_database()
    
    def init_database(self) -> None:
        '''初始化数据库,如果不存在表则创建

        '''
        cur = self.conn.cursor()
        logger.debug(f'初始化数据库')
        
        #创建up表   
        cur.execute('''CREATE TABLE IF NOT EXISTS up
            (
                up_uid VARCHAR(20) PRIMARY KEY NOT NULL,
                up_name VARCHAR(50) NOT NULL,
                latest_update INT NOT NULL
            )
            ''')
        # 创建liver表
        cur.execute('''CREATE TABLE IF NOT EXISTS liver
            (
                liver_uid VARCHAR(20) PRIMARY KEY NOT NULL,
                liver_name VARCHAR(50) NOT NULL,
                is_live TINYINT(1) DEFAULT 0 NOT NULL,
                live_room VARCHAR(30) NOT NULL
            )
            ''')

        # 创建telegram表
        cur.execute('''CREATE TABLE IF NOT EXISTS telegram
            (
                season_id VARCHAR(20) PRIMARY KEY NOT NULL,
                telegram_title VARCHAR(50) NOT NULL,
                episode INT NOT NULL,
                is_finish TINYINT(1) DEFAULT 0 NOT NULL
            )
            ''')
        
        #创建动态表   
        cur.execute('''CREATE TABLE IF NOT EXISTS dynamic
            (
                uid VARCHAR(20) PRIMARY KEY NOT NULL,
                u_name VARCHAR(50) NOT NULL,
                pin_id_str VARCHAR(25) NOT NULL,
                latest_timestamp INT NOT NULL
            )
            ''')

        # 创建个人用户表
        cur.execute('''CREATE TABLE IF NOT EXISTS qq_user
            (
                user_id VARCHAR(20) PRIMARY KEY NOT NULL,
                user_name VARCHAR(50) NOT NULL
            )
            ''')

        # 创建群用户表
        cur.execute('''CREATE TABLE IF NOT EXISTS qq_group
            (
                group_id VARCHAR(20) PRIMARY KEY NOT NULL,
                group_name VARCHAR(50) NOT NULL
            )
            ''')

        # 创建关注up表
        cur.execute('''CREATE TABLE IF NOT EXISTS up_follower
            (
                id INTERGER PRIMARY KEY,
                up_uid VARCHAR(20) NOT NULL,
                user_id VARCHAR(20),
                group_id VARCHAR(20)
            )
            ''')

        # 创建索引
        cur.execute("CREATE INDEX IF NOT EXISTS up_index ON up_follower (up_uid)")
        cur.execute("CREATE INDEX IF NOT EXISTS up_user_index ON up_follower (user_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS up_group_index ON up_follower (group_id)")

        # 创建关注主播表
        cur.execute('''CREATE TABLE IF NOT EXISTS liver_follower
            (
                id INTERGER PRIMARY KEY,
                liver_uid VARCHAR(20) NOT NULL,
                user_id VARCHAR(20),
                group_id VARCHAR(20)
            )
            ''')

        # 创建索引
        cur.execute("CREATE INDEX IF NOT EXISTS liver_index ON liver_follower (liver_uid)")
        cur.execute("CREATE INDEX IF NOT EXISTS liver_user_index ON liver_follower (user_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS liver_group_index ON liver_follower (group_id)")

        # 创建关注节目表
        cur.execute('''CREATE TABLE IF NOT EXISTS telegram_follower
            (
                id INTERGER PRIMARY KEY,
                season_id VARCHAR(20) NOT NULL,
                user_id VARCHAR(20),
                group_id VARCHAR(20)
            )
            ''')

        # 创建索引
        cur.execute("CREATE INDEX IF NOT EXISTS tele_index ON telegram_follower (season_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS tele_user_index ON telegram_follower (user_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS tele_group_index ON telegram_follower (group_id)")
        
        # 创建关注动态表
        cur.execute('''CREATE TABLE IF NOT EXISTS dynamic_follower
            (
                id INTERGER PRIMARY KEY,
                uid VARCHAR(20) NOT NULL,
                user_id VARCHAR(20),
                group_id VARCHAR(20)
            )
            ''')

        logger.debug(f'数据库初始化完毕')
        
        cur.close()
        self.conn.commit()

    def insert_info(self, sql_type: int, *args) -> bool:
        '''插入信息表的通用操作

        Args:
            type (int): 0-用户;1-群组;2-up;3-主播;4-番剧;5-动态
            args: 
                0-用户:   user_id, user_name
                1-群组:   group_id, group_name
                2-up主:   up_uid, up_name, latest_update
                3-主播:   liver_uid, liver_name, is_live, live_room
                4-番剧:   season_id, telegram_title, episode, is_finish
                5-动态:   uid, u_name, pin_id_str, latest_timestamp

        Returns:
            bool: 是否成功
        '''
        cur = self.conn.cursor()

        sqls = [
            "INSERT INTO qq_user(user_id, user_name) VALUES (?, ?)",
            "INSERT INTO qq_group(group_id, group_name) VALUES (?, ?)",
            "INSERT INTO up(up_uid, up_name, latest_update) VALUES (?, ?, ?)",
            "INSERT INTO liver(liver_uid, liver_name, is_live, live_room) VALUES (?, ?, ?, ?)",
            "INSERT INTO telegram(season_id, telegram_title, episode, is_finish) VALUES (?, ?, ?, ?)",
            "INSERT INTO dynamic(uid, u_name, pin_id_str, latest_timestamp) VALUES (?, ?, ?, ?)"
        ]
        #logger.info(f'信息表进行插入')
        
        try:
            cur.execute(sqls[sql_type], args)
        except Exception as e:
            ##logger.error(f'[]插入信息表时发生错误:\n{e}')
            self.conn.rollback()
            cur.close()
            raise BiliDatebaseError(f"数据库插入信息表时发生错误:{e.args[0]}")
        else:
            cur.close()
            self.conn.commit()
            #logger.info(f'插入信息表成功')
            return True
        
    def insert_relation(self, sql_type: int, uid: str, follower_id: str) -> bool:
        '''插入关注表

        Args:
            type (int): 0-个人关注up;1-群关注up;2-个人关注主播;3-群关注主播;4-个人关注番剧;5-群关注番剧;6-个人关注动态;7-群关注动态
            uid (str): 被关注者的id
            follower_id (str): 粉丝的id

        Returns:
            bool: 是否成功
        '''

        sqls = [
            "INSERT INTO up_follower(up_uid, user_id) VALUES (?, ?)",
            "INSERT INTO up_follower(up_uid, group_id) VALUES (?, ?)",
            "INSERT INTO liver_follower(liver_uid, user_id)  VALUES (?, ?)",
            "INSERT INTO liver_follower(liver_uid, group_id) VALUES (?, ?)",
            "INSERT INTO telegram_follower(season_id, user_id) VALUES (?, ?)",
            "INSERT INTO telegram_follower(season_id, group_id) VALUES (?, ?)",
            "INSERT INTO dynamic_follower(uid, user_id) VALUES (?, ?)",
            "INSERT INTO dynamic_follower(uid, group_id) VALUES (?, ?)"
        ]
        cur = self.conn.cursor()
        
        assert 0 <= sql_type < len(sqls), "索引长度错误"
        #logger.info(f'关注表进行插入')
        try:
            cur.execute(sqls[sql_type], (uid, follower_id))
        except Exception as e:
            ##logger.error(f'插入关注表时发生错误:\n{e}')
            cur.close()
            self.conn.rollback()
            raise BiliDatebaseError(f"数据库插入关注表时发生错误:{e.args[0]}")
        else:
            #logger.info(f'插入关注表成功')
            cur.close()
            self.conn.commit()
            return True
            
    def query_info(self, sql_type: int, target_id: str) -> Tuple:
        '''查询信息表通用操作

        Args:
            sql_type (int): 0-用户;1-群组;2-up;3-主播;4-番剧;5-动态
            target_id (str): up/主播/番剧/用户/群/动态发布者的id

        Returns:
            Tuple: 查询结果
        '''
        cur = self.conn.cursor()
        sqls = [
            "SELECT user_id, user_name FROM qq_user WHERE user_id = ?",
            "SELECT group_id, group_name FROM qq_group WHERE group_id = ?",
            "SELECT up_uid, up_name, latest_update FROM up WHERE up_uid = ?",
            "SELECT liver_uid, liver_name, is_live, live_room FROM liver WHERE liver_uid = ?",
            "SELECT season_id, telegram_title, episode, is_finish FROM telegram WHERE season_id = ?",
            "SELECT uid, u_name, pin_id_str, latest_timestamp FROM dynamic WHERE uid = ?"
        ]
        assert 0 <= sql_type < len(sqls), "索引长度错误"
        #logger.info(f'查询信息表中{target_id}的信息')
        
        try:
            cur.execute(sqls[sql_type], (target_id,))
        except Exception as e:
            ##logger.error(f'查询信息表时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询信息表时发生错误:{e.args[0]}")
        else:
            result = cur.fetchone()
            return result

    def query_all(self, sql_type: int) -> List[Tuple]:
        '''获得所有的up/主播/番剧以进行更新检查

        Args:
            sql_type (int): 0-up;1-主播;2-番剧;3-用户;4-群组;5-动态

        Returns:
            List[Tuple]: 查询结果
        '''

        sqls = [
            "SELECT up_uid, up_name, latest_update FROM up",
            "SELECT liver_uid, liver_name, is_live, live_room FROM liver",
            "SELECT season_id, telegram_title, episode, is_finish FROM telegram",
            "SELECT user_id FROM qq_user",
            "SELECT group_id FROM qq_group",
            "SELECT uid, u_name, pin_id_str, latest_timestamp FROM dynamic"
        ]
        
        assert 0 <= sql_type < len(sqls), "索引长度错误"
        cur = self.conn.cursor()

        try:
            cur.execute(sqls[sql_type])
        except Exception as e:
            ##logger.error(f'查询信息表时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询信息表时发生错误:{e.args[0]}")

        else:
            result = cur.fetchall()
            return result

    def query_user_relation(self, sql_type: int, target_id: str) -> List[str]:
        '''查询个人相关的关注

        Args:
            sql_type (int): 
            0-关注up的所有用户;1-用户关注的所有up;
            2-关注主播的所有用户;3-用户关注的所有主播;
            4-关注番剧的所有用户;5-用户关注的所有番剧;
            6-关注动态主的所有用户;7-用户关注的所有动态主
            target_id (str): 用户/up/主播/番剧/动态主的id

        Returns:
            List[str]: 查询结果列表
        '''

        sqls = [
            "SELECT user_id FROM up_follower WHERE up_uid = ?",
            "SELECT up_uid FROM up_follower WHERE user_id = ?",
            "SELECT user_id FROM liver_follower WHERE liver_uid = ?",
            "SELECT liver_uid FROM liver_follower WHERE user_id = ?",
            "SELECT user_id FROM telegram_follower WHERE season_id = ?",
            "SELECT season_id FROM telegram_follower WHERE user_id = ?",
            "SELECT user_id FROM dynamic_follower WHERE uid = ?",
            "SELECT uid FROM dynamic_follower WHERE user_id = ?"
        ]
        assert 0 <= sql_type < len(sqls), "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'查询个人关注')
        try:
            cur.execute(sqls[sql_type], (target_id, ))
        except Exception as e:
            ##logger.error(f'查询个人关注时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询个人关注时发生错误:{e.args[0]}")
        else:
            temp = cur.fetchall()
            if not temp:
                return []
            else:
                result = [i[0] for i in temp if i[0] is not None]
                return result
    
    def query_group_relation(self, sql_type: int, target_id: str) -> List[Tuple]:
        '''查询群相关的关注

        Args:
            sql_type (int): 
            0-关注up的所有群;1-群关注的所有up;
            2-关注主播的所有群;3-群关注的所有主播;
            4-关注番剧的所有群;5-群关注的所有番剧;
            6-关注动态主的所有群;7-群关注的所有动态主
            target_id (str): 群/up/主播/番剧的id

        Returns:
            List[Tuple]: 查询结果
        '''

        sqls = [
            "SELECT group_id FROM up_follower WHERE up_uid = ?",
            "SELECT up_uid FROM up_follower WHERE group_id = ?",
            "SELECT group_id FROM liver_follower WHERE liver_uid = ?",
            "SELECT liver_uid FROM liver_follower WHERE group_id = ?",
            "SELECT group_id FROM telegram_follower WHERE season_id = ?",
            "SELECT season_id FROM telegram_follower WHERE group_id = ?",
            "SELECT group_id FROM dynamic_follower WHERE uid = ?",
            "SELECT uid FROM dynamic_follower WHERE group_id = ?"
        ]
        assert 0 <= sql_type < len(sqls), "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'查询群组关注')
        try:
            cur.execute(sqls[sql_type], (target_id, ))
        except Exception as e:
            ##logger.error(f'查询群组关注时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询群组关注时发生错误:{e.args[0]}")
        else:
            temp = cur.fetchall()
            if not temp:
                return []
            else:
                result = [i[0] for i in temp if i[0] is not None]
                return result

    def query_specified_realtion(self, sql_type: int, user_id: str, target_id: str) -> bool:
        '''查询一对一的关注

        Args:
            sql_type (int): 0 个人-up;1 群组-up;2 个人-主播;3 群组-主播;4 个人-番剧;5 群组-番剧;6 个人-动态;7 群组-动态
            user_id (str): 用户/群组id
            target_id (str): up/主播/番剧/动态主id

        Returns:
            bool: 是否关注
        '''

        sqls = [
            "SELECT user_id, up_uid FROM up_follower WHERE user_id = ? AND up_uid = ?",
            "SELECT group_id, up_uid FROM up_follower WHERE group_id = ? AND up_uid = ?",
            "SELECT user_id, liver_uid FROM liver_follower WHERE user_id = ? AND liver_uid = ?",
            "SELECT group_id, liver_uid FROM liver_follower WHERE group_id = ? AND liver_uid = ?",
            "SELECT user_id, season_id FROM telegram_follower WHERE user_id = ? AND season_id = ?",
            "SELECT group_id, season_id FROM telegram_follower WHERE group_id = ? AND season_id = ?",
            "SELECT user_id, uid FROM dynamic_follower WHERE user_id = ? AND uid = ?",
            "SELECT group_id, uid FROM dynamic_follower WHERE group_id = ? AND uid = ?"
        ]
        
        assert 0 <= sql_type < len(sqls), "索引长度错误"
        
        cur = self.conn.cursor()
        ##logger.info(f'查询一对一关注')
        try:
            cur.execute(sqls[sql_type], (user_id, target_id))
        except Exception as e:
            ##logger.error(f'查询一对一关注时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询一对一关注时发生错误:{e.args[0]}")
        else:
            result = cur.fetchone()
            if result:
                return True
            else:
                return False


    def delete_relation(self, sql_type: int, user_id: str, uid: str) -> bool:
        '''进行取关操作

        Args:
            sql_type (int): 
            0-用户取关up;1-群取关up;
            2-用户取关主播;3-群取关主播;
            4-用户取关番剧;5-群取关番剧;
            6-用户取关动态;7-群取关动态;
            user_id (str): 用户/群id
            uid (str): up/主播/番剧id

        Returns:
            bool: 是否成功
        '''
        sqls = [
            "DELETE FROM up_follower WHERE user_id = ? AND up_uid = ?",
            "DELETE FROM up_follower WHERE group_id = ? AND up_uid = ?",
            "DELETE FROM liver_follower WHERE user_id = ? AND liver_uid = ?",
            "DELETE FROM liver_follower WHERE group_id = ? AND liver_uid = ?",
            "DELETE FROM telegram_follower WHERE user_id = ? AND season_id = ?",
            "DELETE FROM telegram_follower WHERE group_id = ? AND season_id = ?",
            "DELETE FROM dynamic_follower WHERE user_id = ? AND uid = ?",
            "DELETE FROM dynamic_follower WHERE group_id = ? AND uid = ?"
        ]
        assert 0 <= sql_type < len(sqls), "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'进行取关操作,取消{user_id}与{uid}之间的关注关系')
        
        try:
            cur.execute(sqls[sql_type], (user_id, uid))
        except Exception as e:
            ##logger.error(f'取关时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库取关时发生错误:{e.args[0]}")

        else:
            ##logger.info(f'取关成功')
            self.conn.commit()
            return True
        
    def update_info(self, sql_type: int, *args) -> bool:
        '''更新up/主播/番剧信息

        Args:
            sql_type (int): 0-更新up;1-更新主播;2-更新番剧
            args     : 
            0 up-整数时间戳,uid;
            1 主播-True/False, uid;
            2 番剧-整数集数, 是否完结, season_id;
            3 动态-置顶动态id, uid;
            4 动态-最新动态时间戳, uid;

        Returns:
            bool: 是否成功
        '''

        sqls = [
            "UPDATE up SET latest_update = ? WHERE up_uid = ?",
            "UPDATE liver SET is_live = ? WHERE liver_uid = ?",
            "UPDATE telegram SET episode = ?, is_finish = ? WHERE season_id = ?",
            "UPDATE dynamic SET pin_id_str = ? WHERE uid = ?",
            "UPDATE dynamic SET latest_timestamp = ? WHERE uid = ?"
        ]
        assert 0 <= sql_type < len(sqls), "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'更新信息表')
        
        try:
            cur.execute(sqls[sql_type], args)
        except Exception as e:
            ##logger.debug(f'更新信息表时发生错误:\n{e}')
            raise e
        else:
            ##logger.info(f'更新信息表成功')
            self.conn.commit()
            return True

    def delete_info(self, sql_type: int, target_id: str) -> bool:
        '''删除用户/群组/up/主播/番剧/动态主信息

        Args:
            sql_type (int): 0-用户;1-群组;2-up;3-主播;4-番剧;5-动态主信息
            target_id (str): 用户/群组/up/主播/番剧/动态主id       

        Returns:
            bool: 是否成功
        '''

        sqls = [
            "DELETE FROM qq_user WHERE user_id = ?",
            "DELETE FROM qq_group WHERE group_id = ?",
            "DELETE FROM up WHERE up_uid = ?",
            "DELETE FROM liver WHERE liver_uid = ?",
            "DELETE FROM telegram WHERE season_id = ?",
            "DELETE FROM dynamic WHERE uid = ?"
        ]

        cur = self.conn.cursor()
        ##logger.info(f'从信息表中删除{target_id}的信息')
        
        try:
            cur.execute(sqls[sql_type], (target_id,))
        except Exception as e:
            ##logger.error(f'从信息表中删除时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库删除信息时发生错误:{e.args[0]}")

        else:
            ##logger.info(f'删除成功')
            self.conn.commit()
            return True

    def check_dynamic_init(self) -> bool:
        '''检查是否完成动态初始化

        Returns:
            bool: 是否完成
        '''
        cur = self.conn.cursor()
        try:
            cur.execute("SELECT is_dynamic_init FROM bili_sys")
        except Exception as e:
            raise BiliDatebaseError(f"数据库查询是否完成动态初始化时发生错误:{e.args[0]}")
        else:
            result = cur.fetchone()
            if result:
                return True
            else:
                cur.execute("UPDATE bili_sys SET is_dynamic_init = 1")
                self.conn.commit()
                return False
    
    def dbg_check_telegram_follow(self) -> List[Tuple]:

        cur = self.conn.cursor()
        ##logger.info(f'查询群组关注')
        try:
            cur.execute("SELECT rowid, season_id, user_id, group_id FROM telegram_follower")
        except Exception as e:
            ##logger.error(f'查询群组关注时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询群组关注时发生错误:{e.args[0]}")
        else:
            temp = cur.fetchall()
            return temp
        
bili_database = BiliDatabase()


from typing import List
from .db import bili_database
from .exception import BiliDatebaseError
from nonebot.log import logger


class BiliTaskManager():
    def __init__(self) -> None:
        self.up_list = dict()
        self.liver_list = dict()
        self.telegram_list = dict()
        self.dynamic_list = dict()

        self.user_follower_list = dict()
        self.group_follower_list = dict()
    
        self.__up_update_check_ptr__ = 0
        self.__dynamic_update_check_ptr__ = 0

        self.__update_check_len__ = 8

        self.__init_from_database__()
        

    def __init_from_database__(self):
        '''从数据库中进行初始化
        '''
        i = 0
        try:
            up_list = bili_database.query_all(0)
            for up_uid, up_name, latest_timestamp in up_list:
                self.up_list[up_uid] = {
                    "uid": up_uid,
                    "up_name": up_name, 
                    "latest_timestamp": latest_timestamp,
                    "user_follower": set(bili_database.query_user_relation(0, up_uid)),
                    "group_follower": set(bili_database.query_group_relation(0, up_uid))
                }
                if not self.up_list[up_uid]["user_follower"] and not self.up_list[up_uid]["group_follower"]:
                    self.up_list.pop(up_uid)
                    logger.debug(f'{up_uid}已经无人关注')
                else:
                    self.up_list[up_uid]["next_update_check"] = i
                i = (i + 1) % self.__update_check_len__
                    
            liver_list = bili_database.query_all(1)
            for liver_uid, liver_name, is_live, room_id in liver_list:
                self.liver_list[liver_uid] = {
                    "liver_uid": liver_uid, 
                    "liver_name": liver_name,
                    "is_live": is_live,
                    "room_id": room_id,
                    "user_follower": set(bili_database.query_user_relation(2, liver_uid)),
                    "group_follower": set(bili_database.query_group_relation(2, liver_uid))
                }
                if not self.liver_list[liver_uid]["user_follower"] and not self.liver_list[liver_uid]["group_follower"]:
                    self.liver_list.pop(liver_uid)
                    logger.debug(f'{liver_uid}已经无人关注')
                
            telegram_list = bili_database.query_all(2)
            
            for season_id, telegram_title, episode, is_finish in telegram_list:
                
                self.telegram_list[season_id] = {
                    "season_id": season_id,
                    "telegram_title": telegram_title,
                    "episode": episode,
                    "is_finish": bool(is_finish),
                    "user_follower": set(bili_database.query_user_relation(4, season_id)),
                    "group_follower": set(bili_database.query_group_relation(4, season_id))
                }
                if not self.telegram_list[season_id]["user_follower"] and not self.telegram_list[season_id]["group_follower"]:
                    self.telegram_list.pop(season_id)
                    logger.debug(f'{season_id}已经无人关注')
            
            dynamic_list = bili_database.query_all(5)
            for uid, u_name, pin_id_str, latest_timestamp in dynamic_list:
                self.dynamic_list[uid] = {
                    "uid": uid,
                    "u_name": u_name,
                    "pin_id_str": pin_id_str,
                    "latest_timestamp": latest_timestamp,
                    "user_follower": set(bili_database.query_user_relation(6, uid)),
                    "group_follower": set(bili_database.query_group_relation(6, uid))
                }
                if not self.dynamic_list[uid]["user_follower"] and not self.dynamic_list[uid]["group_follower"]:
                    self.dynamic_list.pop(uid)
                    logger.debug(f'{uid}已经无人关注')
                else:
                    self.dynamic_list[uid]["next_update_check"] = i
                i = (i + 1) % self.__update_check_len__

        except Exception as e:
            logger.debug(f'{e}')
            
            logger.error(f'初始化任务管理器失败')
    
    def get_up_check_update_list(self) -> List[str]:
        '''每次触发检查更新的up主

        Returns:
            List[str]: 应检查更新的up和时间戳的列表
        '''
        check_update_list = []
        for up_uid in self.up_list.keys():
            if self.up_list[up_uid]["next_update_check"] == self.__up_update_check_ptr__:
                check_update_list.append(up_uid)
        self.__up_update_check_ptr__ = (self.__up_update_check_ptr__ + 1) % self.__update_check_len__
        return check_update_list
    
    def get_dynamic_check_update_list(self) -> List[str]:
        '''每次触发检查更新的动态

        Returns:
            List[str]: 应检查更新动态主的uid
        '''
        check_update_list = []
        for uid in self.dynamic_list.keys():
            if self.dynamic_list[uid]["next_update_check"] == self.__dynamic_update_check_ptr__:
                check_update_list.append(uid)
        self.__dynamic_update_check_ptr__ = (self.__dynamic_update_check_ptr__ + 1) % self.__update_check_len__
        return check_update_list
    
    def update_up_info(self, up_uid: str, latest_timestamp: int) -> bool:
        '''更新任务管理器以及数据库中的up信息

        Args:
            up_uid (str): up的uid
            latest_timestamp (int): 最新视频时间戳

        Returns:
            bool: 是否成功
        '''
        try:
            bili_database.update_info(0, latest_timestamp, up_uid)
            self.up_list[up_uid]["latest_timestamp"] = latest_timestamp
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n更新主播 <{up_uid}> 时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def update_liver_info(self, liver_uid: str, is_live: bool) -> bool:
        '''更新任务管理器以及数据库中的主播信息

        Args:
            liver_uid (str): 主播的uid
            is_live (int): 是否直播

        Returns:
            bool: 是否成功
        '''
        try:
            bili_database.update_info(1, is_live, liver_uid)
            self.liver_list[liver_uid]["is_live"] = is_live
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n更新主播 <{liver_uid}> 时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def update_telegram_info(self, season_id: str, episode: int, is_finish: bool) -> bool:
        '''更新任务管理器以及数据库中的番剧信息

        Args:
            season_id (str): season_id
            episode (int): 最新集数
            is_finish (bool): 是否完结

        Returns:
            bool: 是否成功
        '''
        try:
            bili_database.update_info(2, episode, is_finish, season_id)
            self.telegram_list[season_id]["is_finish"] = is_finish
            self.telegram_list[season_id]["episode"] = episode
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n更新番剧 <{season_id}> 时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def update_dynamic_pin_id(self, uid: str, pin_id_str: str) -> bool:
        '''更新任务管理器以及数据库中的动态主的置顶id

        Args:
            uid (str): 动态主的uid
            pin_id_str (str): 置顶动态的id

        Returns:
            bool: 是否成功
        '''
        try:
            bili_database.update_info(3, pin_id_str, uid)
            self.dynamic_list[uid]["pin_id_str"] = pin_id_str
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n更新动态主 <{uid}> 时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def update_dynamic_latest_timestamp(self, uid: str, latest_timestamp: int) -> bool:
        '''更新任务管理器以及数据库中的动态主的最新动态时间戳

        Args:
            uid (str): 动态主的uid
            latest_timestamp (int): 最新动态的时间戳

        Returns:
            bool: 是否成功
        '''
        try:
            bili_database.update_info(4, latest_timestamp, uid)
            self.dynamic_list[uid]["latest_timestamp"] = latest_timestamp
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n更新动态主 <{uid}> 时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False

    def add_user_follower(self, target_type: int, target_id: str, user_id: str) -> bool:
        '''_summary_

        Args:
            target_type (int): 0-up主;1-主播;2-番剧;3-动态
            target_id (str): up主/主播/番剧/动态主的id 
            user_id (str): 关注者的id

        Returns:
            bool: 是否成功
        '''
        # 写入数据库,如果不存在于缓存,则从数据库写入缓存
        try:
            bili_database.insert_relation(2 * target_type, target_id, user_id)

            if target_type == 0:
                if target_id not in self.up_list:
                    _, up_name, latest_timestamp = bili_database.query_info(2, target_id)
                    self.up_list[target_id] = {
                        "uid": target_id,
                        "up_name": up_name, 
                        "latest_timestamp": latest_timestamp,
                        "user_follower": set(bili_database.query_user_relation(0, target_id)),
                        "group_follower": set(bili_database.query_group_relation(0, target_id)),
                        "next_update_check": self.__up_update_check_ptr__
                    }
                else:
                    self.up_list[target_id]["user_follower"].add(user_id)
            elif target_type == 1 :
                if target_id not in self.liver_list:
                    _, liver_name, is_live, room_id = bili_database.query_info(3, target_id)
                    self.liver_list[target_id] = {
                        "liver_uid": target_id, 
                        "liver_name": liver_name,
                        "is_live": is_live,
                        "room_id": room_id,
                        "user_follower": set(bili_database.query_user_relation(2, target_id)),
                        "group_follower": set(bili_database.query_group_relation(2, target_id))
                    }
                else:
                    self.liver_list[target_id]["user_follower"].add(user_id)
            elif target_type == 2:
                if target_id not in self.telegram_list:
                    _, telegram_title, episode, is_finish = bili_database.query_info(4, target_id)
                    self.telegram_list[target_id] = {
                        "season_id": target_id,
                        "telegram_title": telegram_title,
                        "episode": episode,
                        "is_finish": bool(is_finish),
                        "user_follower": set(bili_database.query_user_relation(4, target_id)),
                        "group_follower": set(bili_database.query_group_relation(4, target_id))
                    }
                else:
                    self.telegram_list[target_id]["user_follower"].add(user_id)
            elif target_type == 3:
                if target_id not in self.dynamic_list:
                    _, u_name, pin_id_str, latest_timestamp = bili_database.query_info(5, target_id)
                    self.dynamic_list[target_id] = {
                        "uid": target_id,
                        "u_name": u_name,
                        "pin_id_str": pin_id_str,
                        "latest_timestamp": latest_timestamp,
                        "next_update_check": self.__dynamic_update_check_ptr__,
                        "user_follower": set(bili_database.query_user_relation(6, target_id)),
                        "group_follower": set(bili_database.query_group_relation(6, target_id))
                }
                else:
                    self.dynamic_list[target_id]["user_follower"].add(user_id)
            
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n用户<{user_id}>关注<{target_id}>时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False

    def add_group_follower(self, target_type: int, target_id: str, group_id: str) -> bool:
        '''_summary_

        Args:
            target_type (int): 0-up主;1-主播;2-番剧;3-动态
            target_id (str): up主/主播/番剧/动态主的id 
            group_id (str): 关注群的id

        Returns:
            bool: 是否成功
        '''
        # 写入数据库,如果不存在于缓存,则从数据库写入缓存
        try:
            bili_database.insert_relation(2 * target_type + 1, target_id, group_id)

            if target_type == 0:
                if target_id not in self.up_list:
                    _, up_name, latest_timestamp = bili_database.query_info(2, target_id)
                    self.up_list[target_id] = {
                        "uid": target_id,
                        "up_name": up_name, 
                        "latest_timestamp": latest_timestamp,
                        "next_update_check": self.__up_update_check_ptr__,
                        "user_follower": set(bili_database.query_user_relation(0, target_id)),
                        "group_follower": set(bili_database.query_group_relation(0, target_id))
                    }
                else:
                    self.up_list[target_id]["group_follower"].add(group_id)
            elif target_type == 1 :
                if target_id not in self.liver_list:
                    _, liver_name, is_live, room_id = bili_database.query_info(3, target_id)
                    self.liver_list[target_id] = {
                        "liver_uid": target_id, 
                        "liver_name": liver_name,
                        "is_live": is_live,
                        "room_id": room_id,
                        "user_follower": set(bili_database.query_user_relation(2, target_id)),
                        "group_follower": set(bili_database.query_group_relation(2, target_id))
                    }
                else:
                    self.liver_list[target_id]["group_follower"].add(group_id)
            elif target_type == 2:
                if target_id not in self.telegram_list:
                    _, telegram_title, episode, is_finish = bili_database.query_info(4, target_id)
                    self.telegram_list[target_id] = {
                        "season_id": target_id,
                        "telegram_title": telegram_title,
                        "episode": episode,
                        "is_finish": bool(is_finish),
                        "user_follower": set(bili_database.query_user_relation(4, target_id)),
                        "group_follower": set(bili_database.query_group_relation(4, target_id))
                    }
                else:
                    self.telegram_list[target_id]["group_follower"].add(group_id)
            elif target_type == 3:
                if target_id not in self.dynamic_list:
                    _, u_name, pin_id_str, latest_timestamp = bili_database.query_info(5, target_id)
                    self.dynamic_list[target_id] = {
                        "uid": target_id,
                        "u_name": u_name,
                        "pin_id_str": pin_id_str,
                        "latest_timestamp": latest_timestamp,
                        "next_update_check": self.__dynamic_update_check_ptr__,
                        "user_follower": set(bili_database.query_user_relation(6, target_id)),
                        "group_follower": set(bili_database.query_group_relation(6, target_id))
                    }
                else:
                    self.dynamic_list[target_id]["group_follower"].add(group_id)
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n群<{group_id}>关注<{target_id}>时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def remove_user_follower(self, target_type: int, target_id: str, user_id: str) -> bool:
        '''移除个人用户的关注

        Args:
            target_type (int): 0-up主/1-主播/2-番剧/3-动态
            target_id (str): up主/主播/番剧的id 
            user_id (str): 关注者的id

        Returns:
            bool: 是否成功
        '''
        try:
            bili_database.delete_relation(target_type * 2, user_id, target_id)
            if target_type == 0:
                self.up_list[target_id]["user_follower"].remove(user_id)
                if not self.up_list[target_id]["user_follower"] and not self.up_list[target_id]["group_follower"]:
                    self.up_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
            elif target_type == 1:
                self.liver_list[target_id]["user_follower"].remove(user_id)
                if not self.liver_list[target_id]["user_follower"] and not self.liver_list[target_id]["group_follower"]:
                    self.liver_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
            elif target_type == 2:
                self.telegram_list[target_id]["user_follower"].remove(user_id)
                if not self.telegram_list[target_id]["user_follower"] and not self.telegram_list[target_id]["group_follower"]:
                    self.telegram_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
            elif target_type == 3:
                self.dynamic_list[target_id]["user_follower"].remove(user_id)
                if not self.dynamic_list[target_id]["user_follower"] and not self.dynamic_list[target_id]["group_follower"]:
                    self.dynamic_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n用户<{user_id}>取关<{target_id}>时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def remove_group_follower(self, target_type: int, target_id: str, group_id: str) -> bool:
        '''移除个人用户的关注

        Args:
            target_type (int): 0-up主/1-主播/2-番剧/3-动态
            target_id (str): up主/主播/番剧的id 
            group_id (str): 关注者的id

        Returns:
            bool: 是否成功
        '''
        try:
            bili_database.delete_relation(target_type * 2 + 1, group_id, target_id)
            if target_type == 0:
                self.up_list[target_id]["group_follower"].remove(group_id)
                if not self.up_list[target_id]["user_follower"] and not self.up_list[target_id]["group_follower"]:
                    self.up_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
            elif target_type == 1:
                self.liver_list[target_id]["group_follower"].remove(group_id)
                if not self.liver_list[target_id]["user_follower"] and not self.liver_list[target_id]["group_follower"]:
                    self.liver_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
            elif target_type == 2:
                self.telegram_list[target_id]["group_follower"].remove(group_id)
                if not self.telegram_list[target_id]["user_follower"] and not self.telegram_list[target_id]["group_follower"]:
                    self.telegram_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
            elif target_type == 3:
                self.dynamic_list[target_id]["group_follower"].remove(group_id)
                if not self.dynamic_list[target_id]["user_follower"] and not self.dynamic_list[target_id]["group_follower"]:
                    self.dynamic_list.pop(target_id)
                    logger.debug(f'{target_id}已经无人关注')
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n群<{group_id}>取关<{target_id}>时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False

    def add_up_info(self, up_uid: str, up_name: str, latest_timestamp: int) -> bool:
        '''向任务管理器以及数据库中增加up信息

        Args:
            up_uid (str): up的uid
            up_name (str): up的用户名
            latest_timestamp (int): 最新视频的时间戳

        Returns:
            bool: 是否成功
        '''
        try:
            if not bili_database.query_info(2, up_uid):
                bili_database.insert_info(2, up_uid, up_name, latest_timestamp)
            self.up_list[up_uid] = {
                "uid": up_uid,
                "up_name": up_name, 
                "latest_timestamp": latest_timestamp,
                "user_follower": set(bili_database.query_user_relation(0, up_uid)),
                "group_follower": set(bili_database.query_group_relation(0, up_uid)),
                "next_update_check": self.__up_update_check_ptr__
            }
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n创建up主<{up_uid}>信息时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def add_liver_info(self, liver_uid: str, liver_name: str, is_live: bool, room_id: str) -> bool:
        '''向任务管理器以及数据库中增加主播信息

        Args:
            liver_uid (str): 主播的uid
            liver_name (str): 主播的用户名
            is_live (bool): 是否正在直播
            room_id (str): 房间id

        Returns:
            bool: 是否成功
        '''
        try:
            if not bili_database.query_info(3, liver_uid):
                bili_database.insert_info(3, liver_uid, liver_name, is_live, room_id)
            self.liver_list[liver_uid] = {
                "liver_uid": liver_uid, 
                "liver_name": liver_name,
                "is_live": is_live,
                "room_id": room_id,
                "user_follower": set(bili_database.query_user_relation(2, liver_uid)),
                "group_follower": set(bili_database.query_group_relation(2, liver_uid))
            }
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n创建主播<{liver_uid}>信息时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def add_telegram_info(self, season_id: str, telegram_title: str, episode: int, is_finish: bool) -> bool:
        '''向任务管理器以及数据库中增加番剧信息

        Args:
            season_id (str): 番剧的season_id
            telegram_title (str): 番剧名
            episode (int): 最新集数
            is_finish (bool): 是否完结

        Returns:
            bool: 是否成功
        '''
        try:
            if not bili_database.query_info(4, season_id): 
                bili_database.insert_info(4, season_id, telegram_title, episode, is_finish)
            self.telegram_list[season_id] = {
                "season_id": season_id,
                "telegram_title": telegram_title,
                "episode": episode,
                "is_finish": is_finish,
                "user_follower": set(bili_database.query_user_relation(4, season_id)),
                "group_follower": set(bili_database.query_group_relation(4, season_id))
            }
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n创建番剧<{season_id}>信息时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
    
    def add_dynamic_info(self, uid: str, u_name: str, pin_id_str: str, latest_timestamp: int) -> bool:
        '''向任务管理器以及数据库中增加动态主信息

        Args:
            uid (str): uid
            u_name (str): 动态主的用户名
            pin_id_str (str): 置顶动态的id
            latest_timestamp (int): 最新动态的时间戳

        Returns:
            bool: 是否成功
        '''
        try:
            if not bili_database.query_info(5, uid):
                bili_database.insert_info(5, uid, u_name, pin_id_str, latest_timestamp)
            self.dynamic_list[uid] = {
                "uid": uid,
                "u_name": u_name,
                "pin_id_str": pin_id_str,
                "latest_timestamp": latest_timestamp,
                "next_update_check": self.__dynamic_update_check_ptr__,
                "user_follower": set(bili_database.query_user_relation(6, uid)),
                "group_follower": set(bili_database.query_group_relation(6, uid))
            }
            logger.debug(f'in add_dynamic_info {self.dynamic_list[uid]}')
            
            return True
        except BiliDatebaseError as e:
            exception_msg = f'【错误报告】\n创建动态主<{uid}>信息时数据库发生错误\n错误信息: {e}\n'
            logger.error(exception_msg)
            return False
            

bili_task_manager = BiliTaskManager()
logger.debug(f' bili task manager - up_list = {bili_task_manager.up_list}')
logger.debug(f' bili task manager - liver_list = {bili_task_manager.liver_list}')
logger.debug(f' bili task manager - telegram_list = {bili_task_manager.telegram_list}')
logger.debug(f' bili task manager - dynamic_list = {bili_task_manager.dynamic_list}')

            







from .exception import BiliConnectionError, BiliDatebaseError
import asyncio
from typing import Tuple, List
import sys
import traceback
import nonebot
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import Message, MessageSegment
from .basicFunc import *
from .bili_client import bili_client
from .bili_task import bili_task_manager
from datetime import datetime
import json

__PLUGIN_NAME = "[bilibilibot~动态]"

# 视频
async def check_dynamic_update() -> None:
    """
    @description  :
    检查关注动态主是否更新新视频，如果更新则通知用户并更新缓存和数据库
    ---------
    @param  :
    -------
    @Returns  :
    -------
    """
    logger.debug("running check_dynamic_update")
    schedBot = nonebot.get_bot()
    check_dynamic_list = bili_task_manager.get_dynamic_check_update_list()
    #logger.debug(f'{__PLUGIN_NAME}check_dynamic_list = {check_dynamic_list}')
    
    results = await asyncio.gather(
        *[bili_client.get_latest_dynamic(
            uid, 
            bili_task_manager.dynamic_list[uid]["pin_id_str"],
            bili_task_manager.dynamic_list[uid]["latest_timestamp"]
        ) for uid in check_dynamic_list],
        return_exceptions=True
    )
    
    for i in range(len(check_dynamic_list)):
        if isinstance(results[i], tuple):
            uid = check_dynamic_list[i]
            u_name = bili_task_manager.dynamic_list[uid]["u_name"]
            if results[i][0] is True:
                # 修改缓存中的置顶动态信息
                logger.debug(f'{__PLUGIN_NAME}原置顶动态为{bili_task_manager.dynamic_list[uid]["pin_id_str"]}')
                logger.debug(f'{__PLUGIN_NAME}新置顶动态为{results[i][2]}')
                
                
                logger.debug(f'{__PLUGIN_NAME}检测到<{u_name}>置顶动态发生了改变')
                bili_task_manager.update_dynamic_pin_id(uid, results[i][2])
            if results[i][1] is True:
                latest_timestamp = 0
                
                logger.info(f'{__PLUGIN_NAME}检测到<{u_name}>更新了动态')
                logger.debug(json.dumps(results[i][3], ensure_ascii=False, indent=4))

                info_msg = Message()
                text_msg = f"【B站动态】\n<{u_name}>更新了动态:\n"
                info_msg.append(text_msg)
                
                
                for dynamic_item in results[i][3]:
                    latest_timestamp = max(latest_timestamp, dynamic_item["timestamp"])
                    if dynamic_item["major_type"] in ("MAJOR_TYPE_DRAW", "MAJOR_TYPE_NONE", "MAJOR_TYPE_ARTICLE"):
                        dynamic_msg = Message()
                        dynamic_msg.append(MessageSegment.text("=============\n"))

                        # add time
                        dynamic_msg.append(MessageSegment.text(
                            "--" + 
                            datetime.fromtimestamp(dynamic_item["timestamp"]).strftime("%Y-%m-%d %H:%M")
                            + "--\n"))
                        # add text
                        dynamic_msg.append(MessageSegment.text(dynamic_item["text"] + "\n"))

                        # add image
                        for img_src in dynamic_item["image"]:
                            dynamic_msg.append(MessageSegment.image(img_src))
                        
                        # 如果是专栏,添加专栏信息
                        if dynamic_item["tag"] == "MAJOR_ARTICLE":
                            dynamic_msg.append(
                                MessageSegment.text(
                                    f'\n<{u_name}>发布了专栏 《{dynamic_item["article"]["title"]}》\n\
                                    链接: {dynamic_item["article"]["jump_url"]}\n'
                                )
                            )
                            dynamic_msg.append(MessageSegment.image(dynamic_item["article"]["cover"]))
                        # 添加附加信息
                        # 附加信息是投票
                        elif dynamic_item["tag"] == "ADD_VOTE":
                            dynamic_msg.append(MessageSegment.text("----附加信息----\n"))
                            dynamic_msg.append(MessageSegment.text(f'<{u_name}>发布了投票: <{dynamic_item["vote"]["desc"]}>\n'))
                        # 附加信息是视频
                        elif dynamic_item["tag"] == "ADD_UGC":
                            dynamic_msg.append(MessageSegment.text("----附加信息----\n"))
                            dynamic_msg.append(MessageSegment.text(f'标题: {dynamic_item["tgc"]["title"]}\n链接: {dynamic_item["tgc"]["jump_url"]}\n'))
                            dynamic_msg.append(MessageSegment.image(dynamic_item["tgc"]["cover"]))
                        # 附加信息是预约
                        elif dynamic_item["tag"] == "ADD_RESERVE":
                            dynamic_msg.append(MessageSegment.text("----附加信息----\n"))
                            dynamic_msg.append(MessageSegment.text(f'<{u_name}>发布了预约: <{dynamic_item["reserve"]["title"]}>\n{dynamic_item["reserve"]["desc"]}\n'))
                        dynamic_msg.append(MessageSegment.text("=============\n"))
                    
                        info_msg.extend(dynamic_msg)
                bili_task_manager.update_dynamic_latest_timestamp(uid, latest_timestamp)

                if len(info_msg) != 1:
                    user_list = bili_task_manager.dynamic_list[uid]["user_follower"]
                    for user_id in user_list:
                        await schedBot.send_msg(message=info_msg, user_id=user_id)
                    
                    group_list = bili_task_manager.dynamic_list[uid]["group_follower"]
                    for group_id in group_list:
                        await schedBot.send_msg(message=info_msg, group_id=group_id)

        elif isinstance(results[i], (BiliAPIRetCodeError, BiliStatusCodeError, BiliConnectionError)):
            exception_msg = f'[错误报告]\n检测动态主 <{check_dynamic_list[i]}> 更新情况时发生错误\n错误类型: {type(results[i])}\n错误信息: {results[i]}'
            logger.error(f"[{__PLUGIN_NAME}]" + exception_msg)

async def follow_dynamic(uid: str, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据用户或群关注动态主，修改数据库

    Args:
        uid (str): 动态主的uid
        user_id (str): 用户的qq或群号
        user_type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''
    
    # 处理参数错误
    if not uid.isdigit():
        logger.error(f'{__PLUGIN_NAME}存在错误参数<{uid}>')
        return (False, uid + "(错误参数)")
    
    uid = str(int(uid))
    
    try:

        logger.debug(f'{__PLUGIN_NAME}uid = {uid}')
        if uid not in bili_task_manager.dynamic_list:
            logger.debug(f'{__PLUGIN_NAME}uid not in dynamic_list')
        else:
            logger.debug(f'{__PLUGIN_NAME}uid in dynamic_list: {bili_task_manager.dynamic_list[uid]}')
            
            
        if uid not in bili_task_manager.dynamic_list:
            u_name, pin_id_str, latest_timestamp = await bili_client.init_dynamic_info(uid)
            
            bili_task_manager.add_dynamic_info(uid, u_name, pin_id_str, latest_timestamp)
            logger.debug(f'{__PLUGIN_NAME}after update dynamic_list[uid] = {bili_task_manager.dynamic_list[uid]}')
        
            if user_type == 0:
                bili_task_manager.add_user_follower(3, uid, user_id)
            else:
                bili_task_manager.add_group_follower(3, uid, user_id)
            logger.debug(f'{__PLUGIN_NAME}after follow,  dynamic_list[uid] = {bili_task_manager.dynamic_list[uid]}')
        
            logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 关注动态主 <{u_name}> 成功")
            logger.debug(f"{uid}-{u_name}-{pin_id_str}-{latest_timestamp}")
            return (True, u_name + f"(uid: {uid})")
            
        u_name = bili_task_manager.dynamic_list[uid]["u_name"]
        # 处理已关注
        if user_type == 0 and user_id in bili_task_manager.dynamic_list[uid]["user_follower"] or \
            user_type == 1 and user_id in bili_task_manager.dynamic_list[uid]["group_follower"]:
            logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 已经关注了动态主 <{u_name}>")
            return (False, u_name + "(已关注)")

        # 进行关注
        if user_type == 0:
            bili_task_manager.add_user_follower(3, uid, user_id)
        elif user_type == 1:
            bili_task_manager.add_group_follower(3, uid, user_id)
        logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 关注动态主 <{u_name}> 成功")
        return (True, u_name + f"(uid: {uid})")
    except BiliAPI404Error:
        return (False, uid + "(uid错误)")
    except (BiliConnectionError, BiliAPIRetCodeError, BiliStatusCodeError):
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n获取动态主 <{uid}> B站信息发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(网络错误)")
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注动态主 <{uid}>时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(数据库错误)")
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注动态主 <{uid}>时发生意料之外的错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(未知错误,请查看日志)")

async def unfollow_dynamic(uid: str, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据个人用户或群取关动态主，修改up文件

    Args:
        uid (str): 动态主的uid
        user_id (str): 用户的qq号/群号
        user_type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''
    
    # 处理参数错误
    if not uid.isdigit():
        logger.error(f'{__PLUGIN_NAME}存在错误参数 <{uid}>')
        return (False, uid + "(错误参数)")
    try:
    # 处理未关注
        #logger.debug(f"{uid}的关注列表{bili_task_manager.dynamic_list[uid]['user_follower']}")
        if uid not in bili_task_manager.dynamic_list or \
            user_type == 0 and user_id not in bili_task_manager.dynamic_list[uid]["user_follower"] or \
            user_type == 1 and user_id not in bili_task_manager.dynamic_list[uid]["group_follower"]:
            logger.info(f'{__PLUGIN_NAME}用户/群 <{user_id}> 未关注动态主 <{uid}>')
            return (False, uid + "(未关注)")
        
        # 进行取关
        u_name = bili_task_manager.dynamic_list[uid]["u_name"]
        if user_type == 0:
            bili_task_manager.remove_user_follower(3, uid, user_id)
        else:
            bili_task_manager.remove_group_follower(3, uid, user_id)
        logger.info(f'{__PLUGIN_NAME}用户/群 <{user_id}> 取关动态主 <{uid}> 成功')

        return (True, u_name  + f"(uid: {uid})") 
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n取关动态主 <{uid}> 时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(数据库错误)")

async def follow_dynamic_list(
    user_id: int, 
    uid_list: List[str], 
    user_type: int
    ) -> List[List[str]]:
    '''个人用户/群关注动态主

    Args:
        user_id (int): qq号/群号
        uid_list (List[str]): 动态主的uid
        user_type (int): 0-个人用户，1-群        

    Returns:
        List[List[str]]: [[关注成功列表]，[关注失败列表]]
    '''
    successList = []
    failList = []
    
    for uid in uid_list:
        isSuccess, s = await follow_dynamic(uid, str(user_id), user_type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)

    return [successList, failList]

async def unfollow_dynamic_list(
    user_id: int, 
    uid_list: List[str],
    user_type: int
    ) -> List[List[str]]:
    '''个人用户/群取关动态主

    Args:
        user_id (int): qq号/群号
        uid_list (List[str]): 取关的动态主
        user_type (int): 0-个人用户，1-群

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    successList = []
    failList = []

    for uid in uid_list:
        isSuccess, s = await unfollow_dynamic(uid, str(user_id), user_type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)
    
    return [successList, failList]